"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

# -*- coding: utf-8 -*-
from constants import AMC_PATH, ENTERPRISE_FLAG_FILE
from constants import AMC_CONFIG_PATH
from constants import SECRET_KEY
#import ALLOWED_ROLES
import sys
import datetime
import os.path
sys.path.insert(1, '%s/site-packages/' % AMC_PATH)
sys.path.append(str(os.path.dirname(AMC_CONFIG_PATH)))

from flask import Flask
from libs.flask_compress import Compress
from utilities import background_tasks
from utilities import config_manager
#from werkzeug.exceptions import Unauthorized
#import traceback
from flask_kvsession import KVSessionExtension
from simplekv.memory import DictStore

store = DictStore()
app = Flask(__name__)
#app.config["UPDATE_INTERVAL"] = int(config_manager.get_update_interval())
app.secret_key = SECRET_KEY
app.config["COMPRESS_DEBUG"] = True
app.config["COMPRESS_MIMETYPES"] = ["text/javascript", "application/javascript",
     "application/x-javascript", "image/png", "text/html", "text/css"]
KVSessionExtension(store, app)

try:
    from gunicorn_config import certfile, keyfile
    if os.path.exists(certfile) and os.path.exists(keyfile):
        app.config['SESSION_COOKIE_SECURE'] = True
        app.config['SESSION_COOKIE_HTTPONLY'] = False
except ImportError:
    app.config['SESSION_COOKIE_SECURE'] = False
    app.config['SESSION_COOKIE_HTTPONLY'] = True

Compress(app)

from flask import json, request, render_template, session
from controllers import cluster_controller, namespace_controller
from controllers import config_controller, xdr_controller
from controllers import node_controller, udf_controller
from controllers import crud_controller
from controllers import multicluster_controller
from utilities.cluster_manager import ClusterManager
from utilities.user_manager import UserManager
from utilities.session_manager import SessionManager
from utilities.config_manager import config_manager
from constants import *
#from itsdangerous import URLSafeTimedSerializer
#from models.user import User
#from helpers import encryption
from libs import base
from helpers.debugger import Debugger
import eventlet
import uuid

ansible_present = True
try:
    import ansible.runner
    from ansible.errors import AnsibleError
except ImportError:
    ansible_present = False

cluster_manager = ClusterManager()
session_manager = SessionManager()
user_manager = UserManager()
debugger = Debugger()


# URLs for static contents


@app.route("/", methods=["GET"])
def onepage():

    response = app.make_response(render_template('onepage.html'))
    response.headers.add('amc-page', 'homepage')
    return response

"""
# URLs for apis
#@app.route("/logout/<cluster>", methods=["GET"])
@app.route("/session-terminate", methods=["POST"])
#@login_required
@session_manager.check_session_validity(session)
#@cluster_manager.check_cluster_validity()
def logout_page():

    #cluster.remove_user(current_user)
    session_key = session['session']
    clusters = session_manager.get_clusters_from_session(session_key)
    for cluster in clusters:
        ip, port = str(cluster['seed_node']).split(':')
        cluster_obj = cluster_manager.get_cluster_obj_by_node(str(cluster['seed_node']))
        if cluster_obj is not None and cluster['username'] is not None:
            cluster_controller.close_sockets(cluster_obj)
        if cluster['username'] is not None:
            cluster_manager.remove_cluster(cluster['cluster_id'])
    session_manager.delete_session(session_key)
    session.pop('session', None)
    response = {'status': 'success'}
    return json.dumps(response)
"""

"""@app.route("/change_password", methods=["POST"])
#@login_required
@session_manager.check_session_validity(session)
def change_password():

    if 'username' in request.form:
        user = User.get(request.form['username'])
    else:
        user = User.get("admin")

    if user and encryption.hash_pass(request.form['old_password'],
         app.secret_key) == user.password:
        current_users = UserManager()
        status = current_users.change_password(user.id,
             encryption.hash_pass(request.form['new_password'], app.secret_key))
        if status == "success":
            response = {'status': "success"}
            return json.dumps(response)
    response = {'status': "failure"}
    return json.dumps(response)"""


@app.route("/aerospike/service/clusters/<cluster>/change_password", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def change_password(cluster, user, password):

    from libs import citrusleaf as cl
    response = cluster_controller.change_password(cluster,
                        str(request.form['user']),
                        str(cl.hashpassword(str(request.form['old_password']))),
                        str(cl.hashpassword(str(request.form['new_password']))))
    if response['status'] == 'success':
        resp = session_manager.remove_cluster_from_other_sessions(session['session'], user, str(cluster.get_seed()))
        for entry in resp:
            if entry['seed_node'] is not None:
                cluster_obj = cluster_manager.get_cluster_obj_by_node(entry['seed_node'])
                if cluster_obj is not None:
                    cluster_controller.close_sockets(cluster_obj)
                if entry['cluster_id'] is not None:
                    cluster_manager.remove_cluster(entry['cluster_id'])
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get-current-user", methods=["GET"])
@debugger.check_and_log
def get_current_user(cluster):

    if 'session' in session:
        session_key = session['session']
        username = session_manager.get_user(session_key, cluster)
        #username = current_user.get_id()

        response = dict(username=username)
        response['cluster_id'] = cluster
    else:
        response = dict(error='Session Invalidated!!', username=None,
             cluster_id=None)
    return json.dumps(response)


@app.route("/set-update-interval/<cluster>", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("sys-admin")
def set_update_interval(cluster, user, password):

    if int(request.form['update_interval']) < 1 or int(request.form['update_interval']) > 10:
        response = dict(status="invalid")
    else:
        response = config_manager.set_update_interval(request.form['update_interval'])
    return json.dumps(response)


@app.route("/get_amc_version", methods=["GET"])
@debugger.check_and_log
def get_amc_version():

    try:
        f = open('/opt/amc/amc_version', 'r')
        amc_version = f.read()
        f.close()
    except:
        amc_version = "Error reading AMC version"
    amc_type = "enterprise" if os.path.isfile(ENTERPRISE_FLAG_FILE) else "community"
    return json.dumps(dict(amc_version=amc_version.strip(), amc_type=amc_type))


@app.route("/aerospike/service/clusters/get-cluster-id", methods=["POST"])
@debugger.check_and_log
def get_cluster_id():
    cluster_details = dict(
        seed_node=request.form.get('seed_node', None)
    )
    # TODO: Use map and lambda here
    cluster_name = request.form.get('cluster_name', None)
    if cluster_name:
        cluster_details['cluster_name'] = cluster_name

    username = request.form.get('username', None)
    if username:
        cluster_details['username'] = username

    password = request.form.get('password', None)
    if password:
        cluster_details['password'] = password

    response = cluster_controller.get_cluster_id(
        cluster_details, session, session_manager, cluster_manager
    )

    # Setting mc dependency of cluster to be independent
    cluster_id = response.get('cluster_id', None)
    if cluster_id:
        cluster_obj = cluster_manager.get_cluster_by_id(response['cluster_id'])
        cluster_obj.set_mc_dependent(False)

    return json.dumps(response)


@app.route("/aerospike/get_multicluster_view/<xdr_port>", methods=['GET'])
@debugger.check_and_log
def get_multicluster_view(xdr_port):
    response = multicluster_controller.get_info(
        session, session_manager, cluster_manager, xdr_port
    )
    return json.dumps(response)


@app.route("/get_current_monitoring_clusters", methods=["GET"])
@session_manager.check_session_validity(session)
@debugger.check_and_log
def get_current_monitoring_clusters():
    session_key = session['session']
    response = session_manager.get_clusters_from_session(session_key)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/logout", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def delete_cluster_from_session(cluster, user, password):
    session_key = session['session']
    if user is not None:
        cluster_controller.close_sockets(cluster)
        cluster_manager.remove_cluster(cluster.get_cluster_id())
    status = session_manager.remove_cluster(session_key, cluster.get_cluster_id())
    if status:
        response = dict(status="success")
    else:
        response = dict(status="failure")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_cluster_info(cluster, user, password):

    response = cluster_controller.get_info(cluster, user=user,
         password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/basic", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_cluster_basic_info(cluster, user, password):

    response = cluster_controller.get_basic_info(cluster, user=user,
         password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<node_list>", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_node_info(cluster, node_list, user, password):

    response = node_controller.get_info(cluster, node_list,
                                    user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/allstats", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_node_stats(cluster, address, user, password):

    stats_type = request.args.get("type", "all")

    response = node_controller.get_stats(cluster, address, stats_type,
                                user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/allconfig", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_node_config(cluster, address, user, password):

    response = node_controller.get_all_config(cluster, address)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<node_list>/setconfig", methods=["POST"])
@session_manager.check_session_validity(session, ['sys-admin'])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("sys-admin")
def set_node_config(cluster, node_list, user, password):

    edit_parameters = request.form
    response = config_controller.set_node_config(cluster, node_list,
        edit_parameters, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/nodes/<node_list>",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_namespace_info_from_node(cluster, namespace, node_list, user, password):

    response = namespace_controller.get_namespace_info_per_node(cluster,
         str(namespace), node_list, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/nodes/<node>/allstats",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_namespace_stats_from_node(cluster, namespace, node, user, password):

    stats_type = request.args.get("type", "all")

    response = namespace_controller.get_stats(cluster, str(namespace), node,
         stats_type, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace_list>", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_namespace_info(cluster, namespace_list, user, password):

    node_addresses = request.args.get('nodes', None)

    response = namespace_controller.get_namespace_info(cluster,
         str(namespace_list), node_addresses, user=user,
         password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/nodes/<node>/allconfig",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_namespace_config(cluster, namespace, node, user, password):

    response = namespace_controller.get_all_config(cluster, str(namespace),
         node)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace_name>/nodes/<node_list>/setconfig", methods=["POST"])
@session_manager.check_session_validity(session, ['sys-admin'])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("sys-admin")
def set_namespace_config(cluster, namespace_name, node_list, user, password):

    edit_parameters = request.form
    response = config_controller.set_namespace_config(cluster, namespace_name,
         node_list, edit_parameters, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/sindexes/<sindex>/nodes/<node>/allstats",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_secondary_index_stats_from_node(cluster, namespace, sindex, node, user, password):

    stats_type = request.args.get("type", "all")
    response = namespace_controller.get_index_stats(cluster, str(namespace),
         str(sindex), node, stats_type, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/sindexes",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_secondary_indexes_by_namespace(cluster, namespace, user, password):

    response = cluster_controller.get_indexes_by_namespace(cluster,
         str(namespace), user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/sets",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_sets_by_namespace(cluster, namespace, user, password):

    response = cluster_controller.get_sets_by_namespace(cluster, str(namespace),
                                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/storage",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_storage_info(cluster, namespace, user, password):

    response = namespace_controller.get_storage_info(cluster, str(namespace))
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/xdr/<xdr_port>/nodes/<node_list>", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_xdr_info(cluster, xdr_port, node_list, user, password):

    #session_key = session['session']
    #cluster_id = cluster.get_cluster_id()
    #pl_password = session_manager.get_pl_password(session_key, cluster_id)
    response = xdr_controller.get_info(cluster, node_list, xdr_port,
                   user=user, password= password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/xdr/<xdr_port>/nodes/<node>/allstats",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_xdr_stats_from_node(cluster, xdr_port, node, user, password):

    #session_key = session['session']
    #cluster_id = cluster.get_cluster_id()
    #pl_password = session_manager.get_pl_password(session_key, cluster_id)
    stats_type = request.args.get("type", "all")
    response = xdr_controller.get_stats(cluster, node, xdr_port, stats_type,
                   user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/xdr/<xdr_port>/nodes/<node>/allconfig",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_xdr_config(cluster, xdr_port, node, user, password):

    #session_key = session['session']
    #cluster_id = cluster.get_cluster_id()
    #pl_password = session_manager.get_pl_password(session_key, cluster_id)
    response = xdr_controller.get_all_config(cluster, node, xdr_port,
                                user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/xdr/<xdr_port>/nodes/<node_list>/setconfig",
    methods=["POST"])
@session_manager.check_session_validity(session, set(["sys-admin"]))
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def set_xdr_config(cluster, xdr_port, node_list, user, password):

    #session_key = session['session']
    #cluster_id = cluster.get_cluster_id()
    #pl_password = session_manager.get_pl_password(session_key, cluster_id)
    edit_parameters = request.form
    response = config_controller.set_xdr_config(cluster, node_list, xdr_port,
         edit_parameters, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/throughput", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_throughput(cluster, user, password):

    node_addresses = request.args.get('nodes', None)
    response = cluster_controller.get_throughput(cluster, node_addresses,
                                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/throughput_history", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_throughput_history(cluster, user, password):

    start_time = request.args.get('start_time', None)
    end_time = request.args.get('end_time', None)
    if start_time is not None:
        start_time = int(start_time)
    if end_time is not None:
        end_time = int(end_time)
    node_addresses = request.args.get('nodes', None)
    response = cluster_controller.get_throughput_history(cluster,
                                        node_addresses, start_time, end_time,
                                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/latency/<node_list>", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_latency_by_node(cluster, node_list, user, password):

    response = node_controller.get_latency(cluster, str(node_list),
                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/latency_history/<node>", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_latency_history_by_node(cluster, node, user, password):

    start_time = request.args.get('start_time', None)
    end_time = request.args.get('end_time', None)

    response = node_controller.get_latency_history(cluster, str(node),
         start_time, end_time, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters", methods=["GET"])
@debugger.check_and_log
def get_clusters_info():

    response = cluster_controller.get_clusters(cluster_manager)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/alerts", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_cluster_alerts(cluster, user, password):

    last_id = request.args.get('last_id', 0)

    response = cluster_controller.get_alerts(cluster, last_id,
         user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/jobs/nodes/<node>", methods=["GET"])
@session_manager.check_session_validity(session, ["read", "read-write", "read-write-udf","data-admin"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("sys-admin")
def get_scan_jobs(cluster, node, user, password):

    response = udf_controller.get_jobs(cluster, node, user=user,
         password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/udfs", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_udfs(cluster, user, password):

    response = udf_controller.get_udfs(cluster, user=user,
         password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/switch_on",
     methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("sys-admin")
def switch_node_on(cluster, address, user, password):

    # Check for stored credentials
    response = session_manager.get_machine_credentials(
        session.get('session', None), cluster, address)

    machine_username, machine_password, machine_ssh_key = None, None, None

    if response['username']:
        machine_username = response['username']
        if response['password']:
            machine_password = response['password']
        elif response['ssh_key']:
            machine_ssh_key = response['ssh_key']

    # If username and password/ssh_key received from request
    # then update credentials
    if 'username' in request.form or 'password' in request.form or \
            'ssh_key' in request.form:
        machine_username = request.form.get('username', None)
        machine_password = request.form.get('password', None)
        machine_ssh_key = request.form.get('ssh_key', None)

        if machine_ssh_key:
            if not os.path.isfile(machine_ssh_key):
                return json.dumps(
                    {'status': 'Failure', 'error': 'Invalid SSH Key File Path'}
                )

        # Add machine credentials
        session_manager.add_machine_credentials(
            session.get('session', None), cluster, address,
            machine_username, machine_password, machine_ssh_key
        )

    if ansible_present:
        if machine_username and (machine_password or machine_ssh_key):
            response = node_controller.change_node_status(
                "on", cluster, address, machine_username, machine_password,
                machine_ssh_key, usr=user, passwd=password or None)
        else:
            response = node_controller.change_node_status(
                "on", cluster, address, usr=user, passwd=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")

    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/switch_off", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("sys-admin")
def switch_node_off(cluster, address, user, password):

    # Check for stored credentials
    response = session_manager.get_machine_credentials(
        session.get('session', None), cluster, address)

    machine_username, machine_password, machine_ssh_key = None, None, None

    if response['username']:
        machine_username = response['username']
        if response['password']:
            machine_password = response['password']
        elif response['ssh_key']:
            machine_ssh_key = response['ssh_key']

    # If username and password/ssh_key received from request
    # then update credentials
    if 'username' in request.form or 'password' in request.form or \
            'ssh_key' in request.form:
        machine_username = request.form.get('username', None)
        machine_password = request.form.get('password', None)
        machine_ssh_key = request.form.get('ssh_key', None)

        if machine_ssh_key:
            if not os.path.isfile(machine_ssh_key):
                return json.dumps(
                    {'status': 'Failure', 'error': 'Invalid SSH Key File Path'}
                )

        # Add machine credentials
        session_manager.add_machine_credentials(
            session.get('session', None), cluster, address,
            machine_username, machine_password, machine_ssh_key
        )

    if ansible_present:
        if machine_username and (machine_password or machine_ssh_key):
            response = node_controller.change_node_status(
                "off", cluster, address, machine_username, machine_password,
                machine_ssh_key, usr=user, passwd=password or None)
        else:
            response = node_controller.change_node_status(
                "off", cluster, address, usr=user, passwd=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")

    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/switch_xdr_on", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("sys-admin")
def switch_xdr_on(cluster, address, user, password):

    # Check for stored credentials
    response = session_manager.get_machine_credentials(
        session.get('session', None), cluster, address)

    machine_username, machine_password, machine_ssh_key = None, None, None

    if response['username']:
        machine_username = response['username']
        if response['password']:
            machine_password = response['password']
        elif response['ssh_key']:
            machine_ssh_key = response['ssh_key']

    # If username and password/ssh_key received from request
    # then update credentials
    if 'username' in request.form or 'password' in request.form or \
            'ssh_key' in request.form:
        machine_username = request.form.get('username', None)
        machine_password = request.form.get('password', None)
        machine_ssh_key = request.form.get('ssh_key', None)

        if machine_ssh_key:
            if not os.path.isfile(machine_ssh_key):
                return json.dumps(
                    {'status': 'Failure', 'error': 'Invalid SSH Key File Path'}
                )

        # Add machine credentials
        session_manager.add_machine_credentials(
            session.get('session', None), cluster, address,
            machine_username, machine_password, machine_ssh_key
        )

    if ansible_present:
        if machine_username and (machine_password or machine_ssh_key):
            response = xdr_controller.change_xdr_status(
                "on", cluster, address, machine_username, machine_password,
                machine_ssh_key, usr=user, passwd=password or None)
        else:
            response = xdr_controller.change_xdr_status(
                "on", cluster, address, usr=user, passwd=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")

    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/switch_xdr_off", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("sys-admin")
def switch_xdr_off(cluster, address, user, password):

    # Check for stored credentials
    response = session_manager.get_machine_credentials(
        session.get('session', None), cluster, address)

    machine_username, machine_password, machine_ssh_key = None, None, None

    if response['username']:
        machine_username = response['username']
        if response['password']:
            machine_password = response['password']
        elif response['ssh_key']:
            machine_ssh_key = response['ssh_key']

    # If username and password/ssh_key received from request
    # then update credentials
    if 'username' in request.form or 'password' in request.form or \
            'ssh_key' in request.form:
        machine_username = request.form.get('username', None)
        machine_password = request.form.get('password', None)
        machine_ssh_key = request.form.get('ssh_key', None)

        if machine_ssh_key:
            if not os.path.isfile(machine_ssh_key):
                return json.dumps(
                    {'status': 'Failure', 'error': 'Invalid SSH Key File Path'}
                )

        # Add machine credentials
        session_manager.add_machine_credentials(
            session.get('session', None), cluster, address,
            machine_username, machine_password, machine_ssh_key
        )

    if ansible_present:
        if machine_username and (machine_password or machine_ssh_key):
            response = xdr_controller.change_xdr_status(
                "off", cluster, address, machine_username, machine_password,
                machine_ssh_key, usr=user, passwd=password or None)
        else:
            response = xdr_controller.change_xdr_status(
                "off", cluster, address, usr=user, passwd=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")

    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/add_node", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def add_node(cluster, user, password):

    address = request.form['address']
    response = cluster_controller.add_node(cluster, address, user=user,
         password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/remove_node", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def remove_node(cluster, user, password):

    address = request.form['address']
    response = cluster_controller.remove_node(cluster, address)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/initiate_backup", methods=["POST"])
@session_manager.check_session_validity(session, ["read", "read-write", "read-write-udf"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("read", "read-write")
def initiate_backup(cluster, user, password):

    session_key = session['session']
    cluster_id = cluster.get_cluster_id()
    pl_password = session_manager.get_pl_password(session_key, cluster_id)

    if "username" in request.form and "password" in request.form:
        response = cluster_controller.initiate_backup(
            ansible_present, cluster, request.form['namespace'],
            request.form['destination_node_address'],
            request.form['destination_location'], request.form['sets'],
            request.form['only_metadata'], request.form['scan_priority'],
            request.form['terminate_on_change'], request.form['username'],
            request.form['password'], usr=user, passwd=password or None,
            pl_password=pl_password)

    elif "username" in request.form:
        response = cluster_controller.initiate_backup(
            ansible_present, cluster, request.form['namespace'],
            request.form['destination_node_address'],
            request.form['destination_location'], request.form['sets'],
            request.form['only_metadata'], request.form['scan_priority'],
            request.form['terminate_on_change'],
            request.form['username'], usr=user,
            passwd=password or None, pl_password=pl_password)
    else:
        response = cluster_controller.initiate_backup(
            ansible_present, cluster, request.form['namespace'],
            request.form['destination_node_address'],
            request.form['destination_location'],
            request.form['sets'], request.form['only_metadata'],
            request.form['scan_priority'], request.form['terminate_on_change'],
            usr=user, passwd=password or None, pl_password=pl_password)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get_backup_progress", methods=["GET"])
@session_manager.check_session_validity(session, ["read", "read-write", "read-write-udf"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("read", "read-write")
def get_backup_progress(cluster, user, password):
    response = cluster_controller.get_backup_progress(
            ansible_present, cluster)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get_successful_backups", methods=["GET"])
@session_manager.check_session_validity(session, ["read", "read-write", "read-write-udf"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("read", "read-write")
def get_successful_backups(cluster, user, password):

    response = cluster_controller.get_successful_backups(cluster)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get_available_backups", methods=["POST"])
@session_manager.check_session_validity(session, ["read", "read-write", "read-write-udf"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("read", "read-write")
def get_available_backups(cluster, user, password):

    response = cluster_controller.get_available_backups(
        ansible_present, cluster, request.form['source_node_address'],
        request.form['source_location'],
        request.form['username'], request.form['password'])
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/initiate_restore", methods=["POST"])
@session_manager.check_session_validity(session, ["read-write", "read-write-udf"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("read-write")
def initiate_restore(cluster, user, password):

    session_key = session['session']
    cluster_id = cluster.get_cluster_id()
    pl_password = session_manager.get_pl_password(session_key, cluster_id)

    if "username" in request.form and "password" in request.form:
        response = cluster_controller.initiate_restore(
            ansible_present, cluster, request.form['source_node_address'],
            request.form['source_location'], request.form['threads'],
            request.form['namespace'], request.form['missing_records_only'],
            request.form['ignore_generation_num'], request.form['username'],
            request.form['password'], usr=user, passwd=password or None,
            pl_password=pl_password)
    elif "username" in request.form:
        response = cluster_controller.initiate_restore(
            ansible_present, cluster, request.form['source_node_address'],
            request.form['source_location'], request.form['threads'],
            request.form['namespace'], request.form['missing_records_only'],
            request.form['ignore_generation_num'], request.form['username'],
            usr=user, passwd=password or None, pl_password=pl_password)
    else:
        response = cluster_controller.initiate_restore(
            ansible_present, cluster, request.form['source_node_address'],
            request.form['source_location'], request.form['threads'],
            request.form['namespace'], request.form['missing_records_only'],
            request.form['ignore_generation_num'], usr=user,
            passwd=password or None, pl_password=pl_password)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get_restore_progress", methods=["GET"])
@session_manager.check_session_validity(session, ["read-write", "read-write-udf"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
#@user_manager.allowed_for_roles("read-write")
def get_restore_progress(cluster, user, password):

    response = cluster_controller.get_restore_progress(
        ansible_present, cluster, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/add_user", methods=["POST"])
@session_manager.check_session_validity(session, ["user-admin"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def add_user(cluster, user, password):

    role_list = request.form['roles'].split(',')
    roles_list = []
    for role in role_list:
        roles_list.append(str(role))
    from libs import citrusleaf as cl
    response = cluster_controller.add_user(cluster, str(request.form['user']),
                            str(cl.hashpassword(str(request.form['password']))),
                            roles_list)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/user/<username>/remove", methods=["POST"])
@session_manager.check_session_validity(session, ["user-admin"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def remove_user(cluster, username, user, password):

    seed = cluster.get_seed()
    response = cluster_controller.remove_user(cluster, str(username))
    if response.get('status', None) is 'success':
        resp = session_manager.remove_cluster_from_session_containing_user(str(username),str(seed))
        for entry in resp:
            if entry['seed_node'] is not None:
                cluster_obj = cluster_manager.get_cluster_obj_by_node(entry['seed_node'])
                if cluster_obj is not None:
                    cluster_controller.close_sockets(cluster_obj)
                if entry['cluster_id'] is not None:
                    cluster_manager.remove_cluster(entry['cluster_id'])
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/user/<username>/update", methods=["POST"])
@session_manager.check_session_validity(session, ["user-admin"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def update_user(cluster, username, user, password):

    seed = cluster.get_seed()
    role_list = request.form['roles'].split(',')
    old_role_list = request.form['old_roles'].split(',')
    roles_list = []
    old_roles_list = []
    for role in role_list:
        roles_list.append(str(role))
    for role in old_role_list:
        old_roles_list.append(str(role))
    from libs import citrusleaf as cl
    if str(request.form['password']) is '':
        response, status = cluster_controller.update_user(cluster, str(username), roles_list, old_roles_list)
    else:
        response, status =  cluster_controller.update_user(cluster, str(username), roles_list, old_roles_list, str(cl.hashpassword(str(request.form['password']))))
    if status:
        resp = session_manager.remove_cluster_from_session_containing_user(str(username), str(seed))
        for entry in resp:
            if entry['seed_node'] is not None:
                cluster_obj = cluster_manager.get_cluster_obj_by_node(entry['seed_node'])
                if cluster_obj is not None:
                    cluster_controller.close_sockets(cluster_obj)
                if entry['cluster_id'] is not None:
                    cluster_manager.remove_cluster(entry['cluster_id'])
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get_all_users", methods=["GET"])
@session_manager.check_session_validity(session, ["user-admin"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_all_users(cluster, user, password):

    response = cluster_controller.get_all_users(cluster)
    return json.dumps(response)

@app.route("/aerospike/service/clusters/<cluster>/get_all_roles", methods=["GET"])
@session_manager.check_session_validity(session, ["user-admin"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_all_roles(cluster, user, password):

    response = cluster_controller.get_all_roles(cluster)
    return json.dumps(response)

@app.route("/aerospike/service/clusters/<cluster>/roles/<role>/get_role", methods=["GET"])
@session_manager.check_session_validity(session, ["user-admin"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def get_role(cluster, role, user, password):

    response = cluster_controller.get_role(cluster, role)
    return json.dumps(response)

@app.route("/aerospike/service/clusters/<cluster>/roles/<role>/add_role", methods=["POST"])
@session_manager.check_session_validity(session, ["user-admin"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def add_role(cluster, role, user, password):

    privileges = request.form['privileges'].split(',')
    privileges_list = []
    for privilege in privileges:
        privileges_list.append(str(privilege))
    response = cluster_controller.add_role(cluster, str(role), privileges_list)
    return json.dumps(response)

@app.route("/aerospike/service/clusters/<cluster>/roles/<role>/drop_role", methods=["POST"])
@session_manager.check_session_validity(session, ["user-admin"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def drop_role(cluster, role, user, password):

    seed = cluster.get_seed()
    all_users = cluster_controller.get_all_users(cluster)
    response = cluster_controller.drop_role(cluster, str(role))
    if all_users['status'] == 'success':
        users_to_be_removed = []
        for user in all_users['users']:
            if role in user['roles']:
                sys.stderr.write('\nUser appended: ' + str(user['user']))
                users_to_be_removed.append(user['user'])
        for username in users_to_be_removed:
            resp = session_manager.remove_cluster_from_session_containing_user(str(username), str(seed))
            for entry in resp:
                if entry['seed_node'] is not None:
                    cluster_obj = cluster_manager.get_cluster_obj_by_node(entry['seed_node'])
                    if cluster_obj is not None:
                        cluster_controller.close_sockets(cluster_obj)
                    if entry['cluster_id'] is not None:
                        cluster_manager.remove_cluster(entry['cluster_id'])

    return json.dumps(response)

@app.route("/aerospike/service/clusters/<cluster>/roles/<role>/add_privileges", methods=["POST"])
@session_manager.check_session_validity(session, ["user-admin"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def add_privileges(cluster, role, user, password):

    privileges = request.form['privileges'].split(',')
    privileges_list = []
    for privilege in privileges:
        privileges_list.append(str(privilege))
    response = cluster_controller.add_privileges(cluster, str(role), privileges_list)
    return json.dumps(response)

@app.route("/aerospike/service/clusters/<cluster>/roles/<role>/drop_privileges", methods=["POST"])
@session_manager.check_session_validity(session, ["user-admin"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def drop_privileges(cluster, role, user, password):

    privileges = request.form['privileges'].split(',')
    privileges_list = []
    for privilege in privileges:
        privileges_list.append(str(privilege))
    response = cluster_controller.drop_privileges(cluster, str(role), privileges_list)
    return json.dumps(response)

@app.route("/aerospike/service/clusters/<cluster>/roles/<role>/update", methods=["POST"])
@session_manager.check_session_validity(session, ["user-admin"])
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def update_role(cluster, role, user, password):

    seed = cluster.get_seed()
    privileges = request.form['privileges'].split(',')
    old_privileges = request.form['old_privileges'].split(',')
    privileges_list = []
    old_privileges_list = []
    for privilege in privileges:
        privileges_list.append(str(privilege))
    for privilege in privileges_list:
        if privilege == '':
            privileges_list.remove(privilege)
    for old_privilege in old_privileges:
        old_privileges_list.append(str(old_privilege))
    response = cluster_controller.update_role(cluster, str(role), privileges_list, old_privileges_list)
    all_users = cluster_controller.get_all_users(cluster)

    if all_users['status'] == 'success':
        users_to_be_removed = []
        for user in all_users['users']:
            if role in user['roles']:
                users_to_be_removed.append(user['user'])
        for username in users_to_be_removed:
            resp = session_manager.remove_cluster_from_session_containing_user(str(username), str(seed))
            for entry in resp:
                if entry['seed_node'] is not None:
                    cluster_obj = cluster_manager.get_cluster_obj_by_node(entry['seed_node'])
                    if cluster_obj is not None:
                        cluster_controller.close_sockets(cluster_obj)
                    if entry['cluster_id'] is not None:
                        cluster_manager.remove_cluster(entry['cluster_id'])

    return json.dumps(response)

"""
#This method should only be enabled in development environment

@app.route("/debug", methods=["GET"])
def eval_query():
    import pprint
    try:
        result = ''
        eval_string = request.args.get("exp", None)
        
        if eval_string == None :
            return json.dumps("Beware of this method. It might break current instance of AMC.")

        result = eval(eval_string)

        if (str(type(result))).find("class") != -1 :
            result = "{'" + pprint.pformat(result) + "': " + pprint.pformat(result.__dict__) + "}"
        else :
            result = pprint.pformat(result)

        result = result.replace('\n',' ')
        return json.dumps(result)
    except:
        return json.dumps("Unexpected error : " + pprint.pformat(sys.exc_info()[0]))
"""


@app.route("/aerospike/service/clusters/<cluster>/namespace/<namespace>/add_index", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def create_index(cluster, namespace, user, password):

    session_key = session['session']
    if user is not None:
        roles = session_manager.get_user_roles_by_cluster_id(session_key, cluster.get_cluster_id())                    
        if "sys-admin" not in roles and "data-admin" not in roles:
            return json.dumps(dict(error="You don't have enough privileges"))
        response = crud_controller.create_index(cluster, str(request.form['namespace']), str(request.form['set_name']), str(request.form['index_name']), str(request.form['bin_name']), str(request.form['bin_type']), user=user, password=password)
    else:
        response = crud_controller.create_index(cluster, str(request.form['namespace']), str(request.form['set_name']), str(request.form['index_name']), str(request.form['bin_name']), str(request.form['bin_type']))
    resp = cluster_controller.get_indexes_by_namespace(cluster, str(namespace), user=user, password=password or None)
    return json.dumps(response)

@app.route("/aerospike/service/clusters/<cluster>/namespace/<namespace>/drop_index", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def drop_index(cluster, namespace, user, password):

    session_key = session['session']
    if user is not None:
        roles = session_manager.get_user_roles_by_cluster_id(session_key, cluster.get_cluster_id())
        if "sys-admin" not in roles and "data-admin" not in roles:
            return json.dumps(dict(error="You don't have enough privileges"))
        response = crud_controller.drop_index(cluster, str(namespace), str(request.form['index_name']), user=user, password=password)
    else:
        response = crud_controller.drop_index(cluster, str(namespace), str(request.form['index_name']))
    resp = cluster_controller.get_indexes_by_namespace(cluster, str(namespace), user=user, password=password or None)
    return json.dumps(response)

@app.route("/aerospike/service/clusters/<cluster>/add_udf", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def add_udf(cluster, user, password):

    session_key = session['session']
    if user is not None:
        roles = session_manager.get_user_roles_by_cluster_id(session_key, cluster.get_cluster_id())
        if "sys-admin" not in roles and "data-admin" not in roles:
            return json.dumps(dict(error="You don't have enough privileges"))
        response = crud_controller.add_udf(cluster, str(request.form['file_name']), str(request.form['file_contents']), str(request.form['udf_type']), user=user, password=password)
    else:
        response = crud_controller.add_udf(cluster, str(request.form['file_name']), str(request.form['file_contents']), str(request.form['udf_type']))
    resp = udf_controller.get_udfs(cluster, user=user, password=password or None)
    return json.dumps(response)

@app.route("/aerospike/service/clusters/<cluster>/drop_udf", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def drop_udf(cluster, user, password):

    session_key = session['session']
    if user is not None:
        roles = session_manager.get_user_roles_by_cluster_id(session_key, cluster.get_cluster_id())
        if "sys-admin" not in roles and "data-admin" not in roles:
            return json.dumps(dict(error="You don't have enough privileges"))
        response = crud_controller.drop_udf(cluster, str(request.form['file_name']), user=user, password=password)
    else:
        response = crud_controller.drop_udf(cluster, str(request.form['file_name']))
    resp = udf_controller.get_udfs(cluster, user=user, password=password or None)
    return json.dumps(response)

@app.route("/aerospike/service/clusters/<cluster>/fire_cmd", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
@debugger.check_and_log
def fire_cmd(cluster, user, password):

    cmd = request.form['command']
    if user is not None:
        resp = cluster_controller.fire_cmd(cluster, str(cmd), user=user, password=password)
    else:
        resp = cluster_controller.fire_cmd(cluster, str(cmd))
    return json.dumps(resp)

@app.route("/aerospike/service/debug", methods=["GET"])
def get_debug_status():

    
    response = debugger.status()

    debug_address = str(request.remote_addr)
    if response['initiator'] is not None:
        response['isOriginInitiator'] = True if debug_address == response['initiator'] else False

    response['status'] = "success"

    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/debug", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def update_debug_status(cluster, user, password):
    import sys
    service = request.form['service']
    if 'duration' in request.form:
        debug_duration_str = request.form['duration']
        debug_duration = int(float(debug_duration_str))

    debug_address = str(request.remote_addr)

    status = "failure"

    if service == "start":
        debug_user = str(user)
        status = "success" if debugger.start(debug_address, cluster.get_cluster_id(), cluster.get_seed(), cluster.get_nodes().keys(), debug_user, debug_duration) else "failure"

    if service == "stop":
        status = "success" if debugger.stop() else "failure"

    if service == "restart":
        debug_user = str(user)
        status = "success" if debugger.restart(debug_address, cluster.get_cluster_id(), cluster.get_seed(), cluster.get_nodes().keys(), debug_user, debug_duration) else "failure"

    response = debugger.status()

    if hasattr(response, 'initiator'):
        response['isOriginInitiator'] = True if debug_address == response['initiator'] else False

    response['service'] = service
    response['status'] = status

    return json.dumps(response)

@app.after_request
def after_request(response):

    response.headers.add('Access-Control-Allow-Origin', '*')
    response.headers.add('Access-Control-Allow-Credentials', 'true')
    response.headers.add('Access-Control-Max-Age', 10)
    response.headers.add('Access-Control-Allow-Methods', 'GET')
    response.headers.add('Cache-Control',
        'must-revalidate, post-check=0, pre-check=0')
    content_type = response.headers.get('Content-Type')
    if content_type.find("text/html") != -1:
        try:
            homepage = response.headers['amc-page']
        except:
            if response.status_code == 404:
                response.headers['Content-Type'] = "text/html"
            else:    
                response.headers['Content-Type'] = "application/json"
            response.headers['Cache-Control'] += ', no-cache'
            return response
    expiry_time = datetime.datetime.utcnow() + datetime.timedelta(30)
    response.headers["Expires"] = expiry_time.strftime("%a, %d %b %Y %H:%M:%S GMT")
    return response

@app.route("/aerospike/service/clusters/<cluster>/get_user_roles", methods=["GET"])
#@login_required
@session_manager.check_session_validity(session)
def get_user_roles(cluster, user, password):

    session_key = session['session']
    roles = session_manager.get_user_roles_by_cluster_id(session_key, cluster)
    if roles is not None:
        roles = list(roles)
    return json.dumps(dict(rolelist=roles))


app.debug = True
app.config["REMEMBER_COOKIE_DURATION"] = datetime.timedelta(days=14)

# Configuring an application with the flask-login
# refer https://flask-login.readthedocs.org/en/latest/
#login_manager.init_app(app)


e = eventlet.spawn(background_tasks.perform_scheduled_tasks, cluster_manager,
                   app, session_manager)
try:
    f = open('/opt/amc/amc_version', 'r')
    amc_version = f.read()
    f.close()
except:
    amc_version = "Error reading AMC version"
sys.stderr.write("AMC Version: %s\n" % amc_version)

if __name__ == "__main__":

    #Change the duration of how long the Remember Cookie is valid on the users
    #computer.  This can not really be trusted as a user can edit it.
    app.config["REMEMBER_COOKIE_DURATION"] = datetime.timedelta(days=14)

    #Tell the login manager where to redirect users to display the login page
    #login_manager.login_view = "/login/"

    #Setup the login manager.
    login_manager.init_app(app)

    app.run(host='0.0.0.0', port=5001)
